# -*- coding: utf-8 -*-
'''I (mklein) have no idea how to write benchmarks in python. These benchmarks are highly
   doubtful!!!'''

LOREMIPSUM = """Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod\
 tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud\
 exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in\
 reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint\
 occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.""".encode('utf-8')

import urlquote
from urllib.parse import quote as urllib_quote
import timeit

def benchmark_quote_urlquote():
    urlquote.quote(LOREMIPSUM)

def benchmark_quote_urllib():
    urllib_quote(LOREMIPSUM)

if __name__ == "__main__":
    print("This library")
    setup = "from __main__ import benchmark_quote_urlquote"
    expression = "benchmark_quote_urlquote()"
    repetitions = 100000
    duration = timeit.timeit(expression, setup=setup, number = repetitions)
    print("Total duration: ", duration, "Average Duration: ", duration / repetitions)

    print("Urllib")
    setup = "from __main__ import benchmark_quote_urllib"
    expression = "benchmark_quote_urllib()"
    repetitions = 100000
    duration = timeit.timeit(expression, setup=setup, number = repetitions)
    print("Total duration: ", duration, "Average Duration: ", duration / repetitions)


# -*- coding: utf-8 -*-
from urlquote._native import ffi, lib
import six

# This buffer is passed to the C-Interface in order to obtain the quoted string. It will be
# reallocated automatically by `_native_quote` and `_native_unquote`, if its size should not be
# large enough. It is ok to reset this buffer to a smaller value, but it always needs to be a valid
# buffer.
buffer = ffi.new('uint8_t[]', 1)

def _native_quote(value):
    """
    Urlencodes the given bytes
    """
    global buffer
    buffer_len = len(buffer)
    quoted_len = lib.quote(value, len(value), buffer, buffer_len)
    if quoted_len > buffer_len:
        # Our buffer has not been big enough to hold the quoted url. Let's allocate a buffer large
        # enough and try again.
        buffer = ffi.new('uint8_t[]', quoted_len)
        lib.quote(value, len(value), buffer, quoted_len)

    return ffi.string(buffer, quoted_len)

def _native_unquote(value):
    """
    Urldecodes the given bytes
    """
    global buffer
    buffer_len = len(buffer)
    unquoted_len = lib.unquote(value, len(value), buffer, buffer_len)
    if unquoted_len > buffer_len:
        # Our buffer has not been big enough to hold the unquoted url. Let's allocate a buffer large
        # enough and try again.
        buffer = ffi.new('uint8_t[]', unquoted_len)
        lib.quote(value, len(value), buffer, unquoted_len)

    return ffi.string(buffer, unquoted_len)

def quote(value):
    """
    Performs string encoding and urlencodes the given string. Always returns utf-8 encoded bytes.
    """
    if not isinstance(value, six.binary_type):
        if not isinstance(value, six.text_type):
            value = str(value)
        value = value.encode('utf-8')
    
    return _native_quote(value)


def unquote(value):
    """
    Decodes a urlencoded string and performs necessary decoding depending on the used python version.
    """
    if not isinstance(value, six.binary_type):
        if not isinstance(value, six.text_type):
            value = str(value)
        value = value.encode('utf-8')
    
    return _native_unquote(value)


from setuptools import setup

def build_native(spec):
    # build rust library
    build = spec.add_external_build(
        cmd=['cargo', 'build', '--release'],
        path='./rust'
    )

    spec.add_cffi_module(
        module_path='urlquote._native',
        dylib=lambda: build.find_dylib('urlquote', in_path='target/release'),
        header_filename=lambda: build.find_header('native.h', in_path='.'),
        rtld_flags=['NOW', 'NODELETE']
    )

setup(
    name='urlquote',
    packages=['urlquote'],
    zip_safe=False,
    platforms='any',
    setup_requires=['milksnake', 'setuptools_scm'],
    install_requires=['milksnake'],
    use_scm_version=True,
    url='https://github.com/blue-yonder/urlquote',
    milksnake_tasks=[
        build_native
    ],
    author='Blue Yonder',
    author_email='oss@blue-yonder.com',
    description='Fast quoting and unquoting of urls.',
)


# -*- coding: utf-8 -*-
from urlquote import quote, unquote

def test_quote_bytes():
    expected = u'/El%20Ni%C3%B1o/'.encode('utf-8')
    actual = quote(u'/El Niño/'.encode('utf-8'))
    assert(expected == actual)

def test_unquote_bytes():
    expected = u'/El Niño/'.encode('utf-8')
    actual = unquote(u'/El%20Ni%C3%B1o/'.encode('utf-8'))
    assert(expected == actual)

def test_quote_string():
    expected = u'/El%20Ni%C3%B1o/'.encode('utf-8')
    actual = quote(u'/El Niño/')
    assert(expected == actual)

def test_unquote_string():
    expected = u'/El Niño/'.encode('utf-8')
    actual = unquote(u'/El%20Ni%C3%B1o/')
    assert(expected == actual)